@@ -12,11 +12,8 @@ use Exporter;
 use bytes;
 use Try::Catch;
 
-# Use inline C program code.
-use Inline 'C';
-
 # Set the package version. 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 # Base class of this module.
 our @ISA = qw(Exporter);
@@ -65,19 +62,26 @@ my $timeout;
 
 # BEGIN block.
 BEGIN {
+    # Try to use Inline C program code.
+    eval {use Inline 'C'; 1};
     # Try to load Time::HiRes
-    eval {require Time::HiRes1;
-         Time::HiRes1->import ( qw(ualarm) );
+    eval {require Time::HiRes;
+         Time::HiRes->import ( qw(ualarm) );
          1;
     };
     if ($@) {
-        # die "Could not load module Time::HiRes\n";
+        # On error use alarm.
         $mod_alarm = 'alarm';
+        # Timeout is in seconds.
         $timeout = 1;
+        # Set flag.
         $is_alarm = 1;
     } else {
+        # On success use ualarm.
         $mod_alarm = 'ualarm';
+        # Timeout is in microseconds.
         $timeout = 1000000;
+        # Set flag.
         $is_alarm = 0; 
     };
     # Try to load sys/ioctl.ph.
@@ -228,49 +232,70 @@ sub set_cursor_position {
 #                                                                              #
 # Description:                                                                 #
 # Get the response from escape sequence.                                       #
-#                                                                              #
-# system "stty cbreak -echo </dev/tty >/dev/tty 2>&1";                         #
-# system "stty -cbreak echo </dev/tty >/dev/tty 2>&1";                         #
 #------------------------------------------------------------------------------# 
 sub ctlseqs_request {
     # Assign the subroutine arguments to the local variables.
     my ($Ps, $user_timeout) = @_;
-    # Define the signal handler.
-    local $SIG{ALRM} = sub {die "timeout"};
+    # Get id from request.
+    my $id = substr($Ps, 1);
+    # Change the timeout if neccessary.
+    $timeout = (defined $user_timeout) ? $user_timeout : $timeout;
+    if ($is_alarm eq 1) {
+        $timeout = int($timeout / 1000000);
+    };
+    $timeout = ($timeout <= 0) ? 1 : $timeout; 
     # Initialise the local variables.
     my ($width, $height) = undef;
     my $buf = '';
     my $chr = '';
-    # Change the timeout.
-    $timeout = (defined $user_timeout) ? $user_timeout : $timeout;
     # Change the settings of the terminal window.
-    io_off();
-    # Print the escape sequence.
+    system "stty cbreak -echo -echonl -echoke -echoe -isig </dev/tty >/dev/tty 2>&1";
+    # Print the escape sequence to STDOUT.
     print STDOUT "${CSI}${Ps}t";
     # Non blocking reading from STDIN.
+    my $match;
+    my @nums;
     eval {
+        # Define the alarm handler.
+        local $SIG{ALRM} = sub {die "timeout"};
         # Start the alarm.
-        $timeout = ($is_alarm eq 1) ? int($timeout / 1000000) : $timeout;
         eval "$mod_alarm $timeout";
-        # Read chars from STDIN. 
         while (($chr = getc(STDIN)) ne "t") {
             $buf .= $chr;
         };
         # Stop the alarm.
         eval "$mod_alarm 0";
     };
-    if ($buf eq '') {
-        # Set width and height.
-        ($width, $height) = (-1, -1);
+    if ($@) {
+         # print "timedout\n";
+         try {
+             die;
+         } catch {
+             system "stty -cbreak echo echonl echoke echoe isig </dev/tty >/dev/tty 2>&1";
+             return (-3, -3);
+         };
     } else {
-        # Get width and height.
-        my $re = qr/\d+;(\d+)\;(\d+)/;
-        ($width, $height) = ($buf =~ $re) ? ($buf =~ $re) : (-2, -2);
+        if ($buf eq '') {
+            # Set width and height.
+            ($width, $height) = (-1, -1);
+        } else {
+            my $re = qr/^.*(\d+\;\d+\;\d+).*$/;
+            if ($buf =~ $re) {
+                $match = $1;
+                @nums = split(";", $match);
+            };
+            # Get width and height.
+            if ($id eq $nums[0]) { 
+                ($height, $width) = ($nums[1], $nums[2]);
+            } else {
+                ($height, $width) = (-2, -2);
+            };
     };
     # Restore the settings of the terminal window.
-    io_on();
+    system "stty -cbreak echo echonl echoke echoe isig </dev/tty >/dev/tty 2>&1";
     # Return width and height.
-    return ($width, $height);
+    return ($height, $width);
+    };
 };
 
 #------------------------------------------------------------------------------# 
@@ -604,10 +629,10 @@ Terminal::Control - Perl extension with methods for the control of the terminal
   cursor_on();
   cursor_off();   
 
-  window_size_chars([$timeout]);  
-  windows_size_pixels([$timeout]);
-  screen_size_chars([$timeout]); 
-  screen_size_pixels([$timeout]);
+  ($rows, $cols) = window_size_chars([$timeout]);  
+  ($xpix, $ypix) = windows_size_pixels([$timeout]);
+  ($rows, $cols) = screen_size_chars([$timeout]); 
+  ($xpix, $ypix) = screen_size_pixels([$timeout]);
 
 Variables in square brackets in the method call are optional. They are in the
 related method predefined.
